import pyb

myip=[192,168,0,10]
myport=4000
hisport=4000

bouton=pyb.Switch()
led1=pyb.LED(1)
led2=pyb.LED(2)
led3=pyb.LED(3)
led4=pyb.LED(4)

bouton_presse=0

def usr_callback_func():
  global bouton_presse
  bouton_presse += 1

bouton.callback(usr_callback_func)

def octet(serie):
  while True:
    c=serie.read(1)
    if c!=None: break
  return ord(c)

def bin2list(num):
  return [num>>8,num&0xff]

def lire(serie):
  paquet=[]
  while True:
    ascii=octet(serie)
    if ascii==0xc0: break
    if ascii==0xdb:
      ascii=octet(serie)
      if ascii==0xdc: paquet.append(0xc0)
      if ascii==0xdd: paquet.append(0xdb)
    else:
      paquet.append(ascii)
  return paquet

def ecrire(serie,paquet):
  for octet in paquet:
    if octet==0xc0:
      serie.writechar(0xdb)
      serie.writechar(0xdc)
    elif octet==0xdb:
      serie.writechar(0xdb)
      serie.writechar(0xdd)
    else:
      serie.writechar(octet)
  serie.writechar(0xc0)

def analyse(serie,paquet):
  version=paquet[0]>>4
  ihl=paquet[0]&0x0F
  ip_length=paquet[2:4]
  iph_length=ihl*4
  ip_header=paquet[0:iph_length]
  sum=somme(ip_header)
  if sum!=0: print("Bad IPv4 checksum : {:04x}".format(sum)); return False
  ttl=paquet[8]
  protocol=paquet[9]
  s_addr=paquet[12:16]
  d_addr=paquet[16:20]
  if d_addr!=myip: return False
  print('version:'+str(version)+'IHL:'+str(ihl)+'Total Length:'+str(iph_length)+'TTL:'+str(ttl)+'Protocol:'+str(protocol))
  if protocol==0x11:
    udp_header=paquet[iph_length:iph_length+8]
    s1=paquet[iph_length:iph_length+2]
    d1=paquet[iph_length+2:iph_length+4]
    length=paquet[iph_length+4:iph_length+6]
    data=paquet[iph_length+8:len(paquet)]
    sum=somme(s_addr+d_addr+[0x00,protocol]+length+udp_header+data)
    if sum!=0: return False
    dport=d1[0]<<8|d1[1]
    if dport!=myport: return False
    print('Source Port:'+str(s1)+'Dest Port:'+str(d1)+'Length:'+str(length)+'Sum:'+str(sum))
    print('Data:'+str(data))
    if len(data)==1:
      byte=paquet[iph_length+8]
      if byte&0x01==0x01:led1.on()
      if byte&0x10==0x10:led1.off()
      if byte&0x02==0x02:led2.on()
      if byte&0x20==0x20:led2.off()
      if byte&0x04==0x04:led3.on()
      if byte&0x40==0x40:led3.off()
      if byte&0x08==0x08:led4.on()
      if byte&0x80==0x80:led4.off()
      if byte==0xff:
        global bouton_presse
        newdata=[bouton_presse]
        bouton_presse=0
        udp=construireUDP(d_addr,s_addr,d1,bin2list(hisport),newdata)
        ecrire(serie,udp)
  if protocol==0x01:
    icmp_header=paquet[iph_length:iph_length+4]
    icmp_type=paquet[iph_length]
    icmp_code=paquet[iph_length+1]
    data=paquet[iph_length+4:len(paquet)]
    sum=somme(icmp_header+data)
    if sum!=0: return False
    print('Type ICMP:'+str(icmp_type)+'Code:'+str(icmp_code)+'Sum:'+str(sum))
    print('Data:'+str(data))
    icmp_reply=construireICMPv4(d_addr,s_addr,data)
    ecrire(serie,icmp_reply)

def somme (paquet):
  somme=0
  if (len(paquet)%2!=0):paquet.append(0)
  for i in range(0,int(len(paquet)/2)):
    mot=paquet[2*i]<<8 | paquet[2*i+1]
    somme += mot
  while somme>>16:
    somme=(somme>>16)+somme&0xffff
  return (~somme)&0xffff

def construireIPv4(ttl,protocol,source,cible,donnees):
  #IP header
  version=4
  ihl=5
  ser=0
  total_length=ihl*4+len(donnees)
  entetesIP=[ version<<4|ihl, 0x00 ] + bin2list(total_length) + [ 0x00, 0x00, 0x00, 0x00, ttl, protocol, 0x00, 0x00 ] + source + cible
  checksum=bin2list(somme(entetesIP))
  entetesIP[10]=checksum[0]
  entetesIP[11]=checksum[1]
  return entetesIP+donnees

def construireICMPv4(source,cible,donnees):
  #ICMP reply
  type=0
  code=0
  icmp=[type, code, 0x00, 0x00]+donnees
  check=bin2list(somme(icmp))
  icmp[2]=check[0]
  icmp[3]=check[1]
  paquetICMP=construireIPv4(0xff,0x01,source,cible,icmp)
  return paquetICMP

def construireUDP(source,cible,sport,dport,donnees):
  #UDP
  #pseudo-tete
  protocol=17
  taille=8+len(donnees)
  pseudo=source+cible+bin2list(protocol)+bin2list(taille)
  #UDP header
  UDPheader=sport+dport+bin2list(taille)+[0x00, 0x00]
  tout=pseudo+UDPheader+donnees
  sum=bin2list(somme(tout))
  UDP=UDPheader+donnees
  UDP[6]=sum[0]
  UDP[7]=sum[1]
  paquetUDP=construireIPv4(0x40,0x11,source,cible,UDP)
  return paquetUDP


def affiche(paquet):
  col=-1
  for ascii in paquet:
    if col%16==15: print('')
    print("{:02x} ".format(ascii),end='')
    col=col+1
  print('')

print("Hello google\n")
serie=pyb.UART(2,9600)
#paquet=[0x45,0x00,0x00,0x54,0x37,0x1b,0x40,0x00,0x40,0x01,0x82,0x32,0xc0,0xa8,0x00,0x0a,0xc0,0xa8,0x00,0x01,0x08,0x00,0x99,0x24,0x05,$
#ecrire(serie,paquet)

while True:
 paquet=lire(serie)
 affiche(paquet)
 if len(paquet)>20: analyse(serie,paquet)
# Fonction analyse(paquet)
# Si mauvais paquet (mauvaise somme de contrôle) le jetter
# Si mauvais destinataire (adresse IP destination != notre adresse IP) le jetter
# Si paquet IP avec ICMP analyser ICMP
# Si paquet IP avec UDP analyser UDP
# Si paquet ICMP et type "echo request" construire un paquet IP+ICMP+"echo reply" et l'envoyer en SLIP

# Fonction IPv4(taille,s_addr,d_addr,protocole,donnees)
# Fonction ICMPv4(type,code,donnees)


